<?php

namespace Drupal\klaro\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * General settings form for the Klaro! consent manager.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'klaro_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'klaro.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('klaro.settings');

    $form['about'] = [
      '#markup' => $this->t('The Klaro! consent <em>notice</em> briefly informs the user about third-party uses. The consent <em>modal</em> can be used to toggle individual apps or purposes.'),
    ];

    $form['vertical_tabs'] = [
      '#type' => 'vertical_tabs',
      '#title' => $this->t('Library settings'),
    ];

    // General settings.
    $form['general_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('General'),
      '#description' => $this->t('Configure behavior and options for the Klaro! element.'),
      '#group' => 'vertical_tabs',
    ];

    $form['general_settings']['auto_show'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show automatically'),
      '#description' => $this->t('If unchecked, the element must be shown programmatically or using a <a href="#" rel="open-consent-manager">custom link</a>.'),
      '#default_value' => $config->get('auto_show'),
    ];
    $form['general_settings']['must_consent'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Require user action'),
      '#description' => $this->t('Check to immediately display the consent manager modal and prevent site interaction until the user accepts/declines the apps.'),
      '#default_value' => $config->get('library.must_consent'),
    ];

    $form['general_settings']['apps'] = [
      '#type' => 'fieldgroup',
      '#title' => $this->t('Apps'),
      '#tree' => TRUE,
    ];

    $form['general_settings']['apps']['group_by_purpose'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Group by purpose'),
      '#description' => $this->t('Allow the user to enable or disable entire groups of apps at once. This also reduces the space taken up by the modal, which is important especially for websites that use many third-party applications.'),
      '#default_value' => $config->get('library.group_by_purpose'),
    ];
    $form['general_settings']['apps']['process_descriptions'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Verbose app descriptions'),
      '#description' => $this->t('If enabled, all Klaro! app descriptions will be processed. As for now they will get extended by the privacy policy url and the info url.'),
      '#default_value' => $config->get('process_descriptions'),
    ];

    $form['general_settings']['buttons'] = [
      '#type' => 'fieldgroup',
      '#title' => $this->t('Buttons'),
      '#tree' => TRUE,
    ];
    $form['general_settings']['buttons']['accept_all'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Accept all'),
      '#description' => $this->t('If checked, <em>all</em> apps are accepted, instead of only required apps and those enabled by default.'),
      '#default_value' => $config->get('library.accept_all'),
    ];
    $form['general_settings']['buttons']['decline_all'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Decline all'),
      '#default_value' => !$config->get('library.hide_decline_all'),
    ];
    $form['general_settings']['buttons']['learn_more'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Learn more'),
      '#description' => $this->t('Allows the user to customize preferences.'),
      '#default_value' => !$config->get('library.hide_learn_more'),
    ];

    // Storage settings.
    $form['storage_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Storage'),
      '#group' => 'vertical_tabs',
    ];

    $form['storage_settings']['storage_method'] = [
      '#type' => 'select',
      '#title' => $this->t('Storage type'),
      '#description' => $this->t('How Klaro! should store the preferences of the user.'),
      '#required' => TRUE,
      '#options' => [
        'cookie' => $this->t('Cookie'),
        'localStorage' => $this->t('Local storage'),
      ],
      '#default_value' => $config->get('library.storage_method'),
    ];

    $form['storage_settings']['cookie_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Cookie name'),
      '#description' => $this->t('You can customize the name of the cookie that Klaro! uses for storing user consent decisions.'),
      '#default_value' => $config->get('library.cookie_name'),
      '#states' => [
        'visible' => [':input[name="storage_method"]' => ['value' => 'cookie']],
      ],
    ];
    $form['storage_settings']['cookie_expires_after_days'] = [
      '#type' => 'number',
      '#title' => $this->t('Cookie expires after'),
      '#description' => $this->t('You can also set a custom expiration time for the Klaro! cookie.'),
      '#min' => 0,
      '#max' => 365,
      '#step' => 1,
      '#field_suffix' => $this->t('days'),
      '#default_value' => $config->get('library.cookie_expires_after_days'),
      '#states' => [
        'visible' => [':input[name="storage_method"]' => ['value' => 'cookie']],
      ]
    ];
    $form['storage_settings']['cookie_domain'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Cookie domain'),
      '#description' => $this->t('You can change to cookie domain for the consent manager itself. Use this if you want to get consent once for multiple matching domains. If undefined, Klaro! will use the current domain.'),
      '#default_value' => $config->get('library.cookie_domain'),
      '#states' => [
        'visible' => [':input[name="storage_method"]' => ['value' => 'cookie']],
      ]
    ];

    // Consent settings.
    $form['advanced'] = [
      '#type' => 'details',
      '#title' => $this->t('Advanced'),
      '#description' => $this->t('Configure advanced settings.'),
      '#group' => 'vertical_tabs',
    ];

    $form['advanced']['element_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Element ID'),
      '#description' => $this->t('Specify the HTML CSS identifier for the Klaro! container.'),
      '#default_value' => $config->get('library.element_id'),
      '#required' => TRUE,
      // In theory, others are allowed, but we like to KISS and stay sane.
      '#pattern' => '[^0-9-][a-zA-Z0-9_-]*',
    ];

    $form['advanced']['deletable_cookie_domains'] = [
      '#type' => 'textarea',
      '#rows' => 5,
      '#title' => $this->t('Matching cookie domains'),
      '#description' => $this->t('Enter one domain per line for cookie deletion. Leave empty to delete the current domain of the visitor only.'),
      '#default_value' => implode("\n", $config->get('deletable_cookie_domains')),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('klaro.settings');

    $config
      ->set('library.element_id', $form_state->getValue('element_id'))
      ->set('library.storage_method', $form_state->getValue('storage_method'))
      ->set('library.cookie_name', $form_state->getValue('cookie_name'))
      ->set('library.cookie_expires_after_days', $form_state->getValue('cookie_expires_after_days'))
      ->set('library.cookie_domain', $form_state->getValue('cookie_domain'));

    $config
      ->set('library.group_by_purpose', $form_state->getValue(['apps', 'group_by_purpose']))
      ->set('library.must_consent', $form_state->getValue('must_consent'))
      ->set('library.accept_all', !!$form_state->getValue(['buttons', 'accept_all']))
      ->set('library.hide_decline_all', !$form_state->getValue(['buttons', 'decline_all']))
      ->set('library.hide_learn_more', !$form_state->getValue(['buttons', 'learn_more']))

      ->set('auto_show', $form_state->getValue('auto_show'))
      ->set('process_descriptions', $form_state->getValue(['apps', 'process_descriptions']));

    $cookie_domains = array_map('trim', explode("\n", $form_state->getValue('deletable_cookie_domains')));
    $config->set('deletable_cookie_domains', array_filter($cookie_domains));

    $config->save();

    parent::submitForm($form, $form_state);
  }

}
